"""
Example 10
Parallel Execution
file name: Example10.py
Copyright Log Opt Co., Ltd.

Consider the same scenario as in Example 3.
Here, Activity 1 can be processed in parallel up to 3 units.
Find the optimal schedule under this new condition.

"""

from optseq import *

m1=Model()
duration ={1:3, 2:2, 3:2, 4:2, 5:4, 6:4, 7:4, 8:4, 9:11, 10:2 }

res=m1.addResource("worker")
res.addCapacity(0,"inf",3)

act={}
mode={}

for i in duration:
    act[i]=m1.addActivity("Act[{0}]".format(i))
    mode[i]=Mode("Mode[{0}]".format(i),duration[i])
    mode[i].addResource(res,1)
    if i==1:
        mode[i].addParallel(1,1,3)
    act[i].addModes(mode[i])

#temporal (precedense) constraints
m1.addTemporal(act[1],act[9])
for i in range(5,9):
    m1.addTemporal(act[4],act[i])
    m1.addTemporal(act[i],act[10])

m1.Params.TimeLimit=1
m1.Params.OutputFlag=True
m1.Params.Makespan=True
m1.optimize()
m1.write("chart10.txt")

##output:
##resource worker
## interval 0 inf capacity 3
##activity Act[1]
##mode duration 3
## worker interval 0 inf requirement 1
## parallel interval 1 1 max 3
##activity Act[2]
##mode duration 2
## worker interval 0 inf requirement 1
##activity Act[3]
##mode duration 2
## worker interval 0 inf requirement 1
##activity Act[4]
##mode duration 2
## worker interval 0 inf requirement 1
##activity Act[5]
##mode duration 4
## worker interval 0 inf requirement 1
##activity Act[6]
##mode duration 4
## worker interval 0 inf requirement 1
##activity Act[7]
##mode duration 4
## worker interval 0 inf requirement 1
##activity Act[8]
##mode duration 4
## worker interval 0 inf requirement 1
##activity Act[9]
##mode duration 11
## worker interval 0 inf requirement 1
##activity Act[10]
##mode duration 2
## worker interval 0 inf requirement 1
##
##temporal Act[1] Act[9] type CS delay 0
##temporal Act[4] Act[5] type CS delay 0
##temporal Act[5] Act[10] type CS delay 0
##temporal Act[4] Act[6] type CS delay 0
##temporal Act[6] Act[10] type CS delay 0
##temporal Act[4] Act[7] type CS delay 0
##temporal Act[7] Act[10] type CS delay 0
##temporal Act[4] Act[8] type CS delay 0
##temporal Act[8] Act[10] type CS delay 0
##
##activity sink duedate 0
##
### reading data ... done: 0.00(s)

### random seed: 1

### tabu tenure: 0

### cpu time limit: 1.00(s)

### iteration limit: 1073741823

### computing all-pairs longest paths and strongly connected components ... done

###scc 12

##objective value = 14 (cpu time = 0.00(s), iteration = 0)

##0: 0.00(s): 14/14

##objective value = 13 (cpu time = 0.01(s), iteration = 25)

##

##--- best activity list ---

##source Act[4] Act[5] Act[1] Act[2] Act[9] Act[6] Act[8] Act[7] Act[3] Act[10] sink

##

##--- best solution ---

##source,---, 0 0

##sink,---, 13 13

##Act[1],---, 0 0--1[2] 1--2 2

##Act[2],---, 1 1--3 3

##Act[3],---, 10 10--12 12

##Act[4],---, 0 0--2 2

##Act[5],---, 2 2--6 6

##Act[6],---, 3 3--7 7

##Act[7],---, 7 7--11 11

##Act[8],---, 6 6--10 10

##Act[9],---, 2 2--13 13

##Act[10],---, 11 11--13 13

##--- tardy activity ---

##sink: 13

##--- resource residuals ---

##worker: [0,12] 0 [12,13] 1

##

##objective value = 13

##cpu time = 0.01/1.00(s)

##iteration = 25/6292

##
##
##source --- 0 0
##sink --- 13 13
##Act[1] --- 0 2
##Act[2] --- 1 3
##Act[3] --- 10 12
##Act[4] --- 0 2
##Act[5] --- 2 6
##Act[6] --- 3 7
##Act[7] --- 7 11
##Act[8] --- 6 10
##Act[9] --- 2 13
##Act[10] --- 11 13
##planning horizon= 13

#chart10.txt
##  activity    mode   duration | 1| 2| 3| 4| 5| 6| 7| 8| 9|10|11|12|13|
##---------------------------------------------------------------------|
## Act[10]   Mode[10]     2     |  |  |  |  |  |  |  |  |  |  |  |==|==|
##  Act[1]   Mode[1]      3     |*2|==|  |  |  |  |  |  |  |  |  |  |  |
##  Act[2]   Mode[2]      2     |  |==|==|  |  |  |  |  |  |  |  |  |  |
##  Act[3]   Mode[3]      2     |  |  |  |  |  |  |  |  |  |  |==|==|  |
##  Act[4]   Mode[4]      2     |==|==|  |  |  |  |  |  |  |  |  |  |  |
##  Act[5]   Mode[5]      4     |  |  |==|==|==|==|  |  |  |  |  |  |  |
##  Act[6]   Mode[6]      4     |  |  |  |==|==|==|==|  |  |  |  |  |  |
##  Act[7]   Mode[7]      4     |  |  |  |  |  |  |  |==|==|==|==|  |  |
##  Act[8]   Mode[8]      4     |  |  |  |  |  |  |==|==|==|==|  |  |  |
##  Act[9]   Mode[9]      11    |  |  |==|==|==|==|==|==|==|==|==|==|==|
##---------------------------------------------------------------------
##   resource usage/capacity    |
##---------------------------------------------------------------------
##            worker            | 3| 3| 3| 3| 3| 3| 3| 3| 3| 3| 3| 3| 2|
##                              | 3| 3| 3| 3| 3| 3| 3| 3| 3| 3| 3| 3| 3|

