"""
Example 12
Example with state
file name: Example12.py
Copyright Log Opt Co., Ltd.

Consider a problem with 2 types of activities.
type1 has 3 activites, 11, 12, 13;
type2 has 3 activites, 21, 22, 23;

The processing times (durations) of the activities are kept in the dictionary
duration ={(1,1):3,(1,2):3,(1,3):3,(2,1):3,(2,2):3,(2,3):3}.

Setup time between 2 different types of activities is 5, for example setup from type1 to type2 needs 5 units of time.
Setup time between the same types of activities is 1.

The objective is to find the maximum completion time (makespan) for all 6 activities
"""
from optseq import *
m1=Model()

duration={(1,1):3,(1,2):3,(1,3):3,(2,1):3,(2,2):3,(2,3):3}
setup={(1,1):1,(1,2):5,(2,1):5,(2,2):1}

act={}
act_setup={}
mode={}
mode_setup={}
rs=m1.addResource("machine",1)
s1=m1.addState("Setup_State")
s1.addValue(time=0,value=1)

#setup mode
for (i,j) in setup:
    mode_setup[i,j]=Mode("Mode_setup"+str(i)+str(j),setup[i,j])
    mode_setup[i,j].addState(s1,i,j)
    mode_setup[i,j].addResource(rs,{(0,"inf"):1})

for (i,j) in duration:
    act_setup[i,j]=m1.addActivity("Setup"+str(i)+str(j),autoselect=True)
    act_setup[i,j].addModes(mode_setup[1,i],mode_setup[2,i])
    act[i,j]=m1.addActivity("Act"+str(i)+str(j))
    mode[i,j]=Mode("Mode"+str(i)+str(j),duration[i,j])
    mode[i,j].addResource(rs,{(0,"inf"):1})
    mode[i,j].addBreak(0,0)
    mode[i,j].addResource(rs,{(0,"inf"):1},"break")
    act[i,j].addModes(mode[i,j])

#temporal (precedense) constraints
for (i,j) in duration:
    m1.addTemporal(act_setup[i,j],act[i,j],"CS")
    m1.addTemporal(act[i,j],act_setup[i,j],"SC")

print(m1)
#m1.Params.Initial="Example12initial.txt"
m1.Params.TimeLimit=1
m1.Params.OutputFlag=True
m1.Params.Makespan=True
m1.optimize()
m1.write("chart12.txt")



