"""
Example 2
PERT with resource constraint
file name: Example2.py
Copyright Log Opt Co., Ltd.

Consider a 5-activity problem with precedence constraints between the activities.
The processing times (durations) of the activities are kept in the dictionary
 duration ={1:13, 2:25, 3:15, 4:27, 5:22 }.
Precedence constraints are given by:
 Activity 1 -> Activity 3; Activity 2 -> Activity 4;
 Activity 3 -> Activity 4; and Activity 3 -> Activity 5.
Each activity requires one unit of worker resource whose capacity (maximum amount of availability for each time period) is 1.
The objective is to find the maximum completion time (makespan) for all 5 activities
"""
from optseq import *
m1=Model()                
duration ={1:10, 2:20, 3:10, 4:25, 5:22 }
#res=m1.addResource("worker",capacity={(0,"inf"):1})
res=m1.addResource("worker",capacity=1)

act={}
mode={}        
for i in duration: 
    act[i]=m1.addActivity("Act[{0}]".format(i))
    mode[i]=Mode("Mode[{0}]".format(i),duration[i])
    mode[i].addResource(res,requirement=1)
    act[i].addModes(mode[i])

#temporal (precedense) constraints
m1.addTemporal(act[1],act[3])
m1.addTemporal(act[2],act[4])
m1.addTemporal(act[2],act[5])
m1.addTemporal(act[3],act[4])

###作業1の開始時刻を計画期間開始時刻の10分後に固定する制約
#m1.addTemporal('source',act[1],'SS',10)
#m1.addTemporal(act[1],'source','SS',-10)

###作業2は作業1終了5分～10分の間に開始しなければならないことを表す制約
#m1.addTemporal(act[1],act[2],'CS',5)
#m1.addTemporal(act[2],act[1],'SC',-10)


m1.Params.TimeLimit=1
m1.Params.OutputFlag=True
m1.Params.Makespan=True
m1.optimize()
m1.write("chart2.txt")

