"""
Example 3
Parallel Shop Problem
file name: Example3.py
Copyright Log Opt Co., Ltd.

Consider a 10-activity problem in which each activity is processed on three identical parallel machines.
The processing times (durations) of the activities are kept in the dictionary
 duration ={1:3, 2:2, 3:2, 4:2, 5:4, 6:4, 7:4, 8:4, 9:11, 10:2 }.
Precedence constraints are given by:
 Activity 1 -> Activity 9;
 Activities 5,6,7,8 are processed after Activity 4 and before Activity 10.
The objective is to find the maximum completion time (makespan).
"""

from optseq import *
m1=Model()
duration ={1:3, 2:2, 3:2, 4:2, 5:4, 6:4, 7:4, 8:4, 9:11, 10:2 }
res=m1.addResource("worker",capacity={(0,"inf"):3})
act={}
mode={}
for i in duration:
    act[i]=m1.addActivity("Act[{0}]".format(i))
    mode[i]=Mode("Mode[{0}]".format(i),duration[i])
    mode[i].addResource(res,{(0,"inf"):1})
    act[i].addModes(mode[i])

#temporal (precedense) constraints
m1.addTemporal(act[1],act[9])
for i in range(5,9):
    m1.addTemporal(act[4],act[i])
    m1.addTemporal(act[i],act[10])

print (m1)
m1.Params.TimeLimit=1
m1.Params.Makespan=True
m1.Params.OutputFlag=True
m1.optimize()
m1.write("chart3.txt")

#output
##source --- 0 0
##sink --- 14 14
##Act[1] --- 0 3
##Act[2] --- 0 2
##Act[3] --- 0 2
##Act[4] --- 2 4
##Act[5] --- 8 12
##Act[6] --- 4 8
##Act[7] --- 8 12
##Act[8] --- 4 8
##Act[9] --- 3 14
##Act[10] --- 12 14
##planning horizon= 14

#chart3.txt
##  activity    mode   duration | 1| 2| 3| 4| 5| 6| 7| 8| 9|10|11|12|13|14|
##------------------------------------------------------------------------|
## Act[10]   Mode[10]     2     |  |  |  |  |  |  |  |  |  |  |  |  |==|==|
##  Act[1]   Mode[1]      3     |==|==|==|  |  |  |  |  |  |  |  |  |  |  |
##  Act[2]   Mode[2]      2     |==|==|  |  |  |  |  |  |  |  |  |  |  |  |
##  Act[3]   Mode[3]      2     |==|==|  |  |  |  |  |  |  |  |  |  |  |  |
##  Act[4]   Mode[4]      2     |  |  |==|==|  |  |  |  |  |  |  |  |  |  |
##  Act[5]   Mode[5]      4     |  |  |  |  |  |  |  |  |==|==|==|==|  |  |
##  Act[6]   Mode[6]      4     |  |  |  |  |==|==|==|==|  |  |  |  |  |  |
##  Act[7]   Mode[7]      4     |  |  |  |  |  |  |  |  |==|==|==|==|  |  |
##  Act[8]   Mode[8]      4     |  |  |  |  |==|==|==|==|  |  |  |  |  |  |
##  Act[9]   Mode[9]      11    |  |  |  |==|==|==|==|==|==|==|==|==|==|==|
##------------------------------------------------------------------------
##   resource usage/capacity    |
##------------------------------------------------------------------------
##            worker            | 3| 3| 2| 2| 3| 3| 3| 3| 3| 3| 3| 3| 2| 2|
##                              | 3| 3| 3| 3| 3| 3| 3| 3| 3| 3| 3| 3| 3| 3|
##------------------------------------------------------------------------







