"""
Example 4
Parallel Shop Problem using Modes
file name: Example4.py
Copyright Log Opt Co., Ltd.

Consider a 10-activity problem in which each activity is processed on three identical parallel machines.
The processing times (durations) of the activities are kept in the dictionary
 duration ={1:3, 2:2, 3:2, 4:2, 5:4, 6:4, 7:4, 8:4, 9:11, 10:2 }.
Precedence constraints are given by:
 Activity 1 -> Activity 9;
 Activities 5,6,7,8 are processed after Activity 4 and before Activity 10.
Activity 1 can be processed in one of the following three modes:
 Mode 1 with duration 3 that requires 1 unit of worker resource,
 Mode 2 with duration 2 that requires 2 units of worker resource, and
 Mode 3 with duration 1 that requires 3 units of worker resource.
The objective is to find the maximum completion time (makespan).
"""

from optseq import *

m1=Model()
duration ={1:3, 2:2, 3:2, 4:2, 5:4, 6:4, 7:4, 8:4, 9:11, 10:2 }
res=m1.addResource("worker",capacity=3)
act={}
mode={}

for i in duration:
    act[i]=m1.addActivity("Act[{0}]".format(i))
    if i==1:
        mode[1,1]=Mode("Mode[1_1]",3)
        mode[1,1].addResource(res,1)
        mode[1,2]=Mode("Mode[1_2]",2)
        mode[1,2].addResource(res,2)
        mode[1,3]=Mode("Mode[1_3]",1)
        mode[1,3].addResource(res,3)
        act[i].addModes(mode[1,1],mode[1,2],mode[1,3])
    else:
        mode[i]=Mode("Mode[{0}]".format(i),duration[i])
        mode[i].addResource(res,1)
        act[i].addModes(mode[i])
#temporal (precedense) constraints
m1.addTemporal(act[1],act[9])
for i in range(5,9):
    m1.addTemporal(act[4],act[i])
    m1.addTemporal(act[i],act[10])

m1.Params.TimeLimit=1
m1.Params.OutputFlag=True
m1.Params.Makespan=True
m1.optimize()
m1.write("chart4.txt")

#output
##source --- 0 0
##sink --- 13 13
##Act[1] Mode[1_3] 0 1
##Act[2] --- 1 3
##Act[3] --- 11 13
##Act[4] --- 1 3
##Act[5] --- 7 11
##Act[6] --- 3 7
##Act[7] --- 7 11
##Act[8] --- 3 7
##Act[9] --- 1 12
##Act[10] --- 11 13
##planning horizon= 13

#chart4.txt
##  activity    mode   duration | 1| 2| 3| 4| 5| 6| 7| 8| 9|10|11|12|13|
##---------------------------------------------------------------------|
## Act[10]   Mode[10]     2     |  |  |  |  |  |  |  |  |  |  |  |==|==|
##  Act[1]  Mode[1_3]     1     |==|  |  |  |  |  |  |  |  |  |  |  |  |
##  Act[2]   Mode[2]      2     |  |==|==|  |  |  |  |  |  |  |  |  |  |
##  Act[3]   Mode[3]      2     |  |  |  |  |  |  |  |  |  |  |  |==|==|
##  Act[4]   Mode[4]      2     |  |==|==|  |  |  |  |  |  |  |  |  |  |
##  Act[5]   Mode[5]      4     |  |  |  |  |  |  |  |==|==|==|==|  |  |
##  Act[6]   Mode[6]      4     |  |  |  |==|==|==|==|  |  |  |  |  |  |
##  Act[7]   Mode[7]      4     |  |  |  |  |  |  |  |==|==|==|==|  |  |
##  Act[8]   Mode[8]      4     |  |  |  |==|==|==|==|  |  |  |  |  |  |
##  Act[9]   Mode[9]      11    |  |==|==|==|==|==|==|==|==|==|==|==|  |
##---------------------------------------------------------------------
##   resource usage/capacity    |
##---------------------------------------------------------------------
##            worker            | 3| 3| 3| 3| 3| 3| 3| 3| 3| 3| 3| 3| 2|
##                              | 3| 3| 3| 3| 3| 3| 3| 3| 3| 3| 3| 3| 3|
##---------------------------------------------------------------------



